miclust <-
function(data, method = "kmeans", search = "none", ks = 2:3, maxVars =  NULL, usedImp = "all", distance = "manhattan", centPos = "colMeans", seed = 4321, initCl = "hc")
 {
  if (!inherits(data, "miData") && !inherits(data, "data.frame"))
     stop("argument 'data' must be of class 'miData' or 'data.frame'")
     
  if (method != "kmeans")
     stop("argument 'method' must be 'kmeans'")
   
  if ((search != "none") && (search != "backward") && (search != "forward"))
     stop("argument 'search' must be 'none', 'backward' or 'forward'")

  if ((initCl != "hc") && (initCl != "rand"))
     stop("argument 'initCl' must be 'hc' or 'rand'")
     
  if ((!inherits(ks, "numeric") && !inherits(ks, "integer")) || is.na(ks[1]))
     stop("argument 'ks' must be numeric")
  
  ks <- as.vector(ks)
  
  if (sum(floor(ks) != ceiling(ks)) > 0)
     stop("values in 'ks' must be integer")
  
  ks <- ks[!duplicated(ks)]
  ks <- ks[order(ks)]
  
  if (max(ks) < 2)
     stop("at least 2 cluster are required in argument 'ks'")
     
  ks <- ks[ks >= 2]
  nk <- length(ks)


  if ((distance != "manhattan") && (distance != "euclidean"))
     stop("argument 'distance' must be 'manhattan' or 'euclidean'")

  if ((centPos != "colMeans") && (centPos != "colMedians"))
     stop("argument 'centPos' must be 'colMeans' or 'colMedians'") 
      
  if (inherits(data, "miData"))
   {
   	data0 <- data$rawData } else {
   	data0 <- data	
   }
  n <- nrow(data0)
  p <- ncol(data0)
    
  if ((search == "forward") &&  ((length(maxVars) != 1L) || is.na(maxVars) || (maxVars < 1) || (floor(maxVars) != ceiling(maxVars)) || (maxVars > p)))
     stop(paste("argument 'maxVars' must be a positive integer not greater than", p, "if search is 'forward'"))
     
   if (inherits(data, "miData"))
   {
   	bases <- data$impData   
    M <- length(bases)*(!is.na(bases))[1] } else {
   	M <- 0
   	usedImp <- 0	
   }
  
  if ((M > 0) && (!inherits(usedImp, "numeric")) && (!inherits(usedImp, "integer")) && (!inherits(usedImp, "character")))
       stop("argument 'usedImp' must be either numeric, integer or character")

  if ((M > 0) && (inherits(usedImp, "character")) && (usedImp != "all"))
       stop("if argument 'usedImp' is character, must be equal to 'all'")
  
  if (M > 0 && usedImp == "all")
    usedImp <- 1:M
    
  if (M > 0 && usedImp != "all")
    wichUsedImp <- usedImp %in% 1:M
    
  if (M == 0 || sum(wichUsedImp) == 0)
    {
     usedImp <- 0 } else {
     usedImp <- usedImp[wichUsedImp]	
    }
  
  if (method == "kmeans")
     {
      eval(parse(text = paste("selMetricCent <- kccaFamily(dist = ", distance, ", cent = ", centPos, ")", sep = "")))
      if (M == 0 || usedImp == 0)
       {
   	    idcomplete <- complete.cases(data0)
   	    completeCasesPerc <- 100*sum(idcomplete)/length(idcomplete) 
   	    aux <- standardizeDataBase(data0[idcomplete, ])
   	    clustering <- list()
   	    CritCF <- vector()
   	    if (search != "none")
   	     NumberOfSelectedVars <- vector()
        for (i in 1:nk)
         {
          cat("k =", ks[i], "\n")
          clustering[[i]] <- doClusterkmeans(search = search, DataBase = aux, k = ks[i], metricCent = selMetricCent, inertiaPower = 1, maxVars = maxVars, centPos = centPos, initCl = initCl, seed = seed)
          auxclust <- rep(NA, nrow(aux))
          auxclust[idcomplete] <- clustering[[i]]$ClusterVector
          clustering[[i]]$ClusterVector <- auxclust
          CritCF[i] <- clustering[[i]]$CritCFmax
          if (search != "none")
           NumberOfSelectedVars[i] <- length(clustering[[i]]$SelectedVariables)   
         }
        names(clustering) <- paste("k=", ks, sep = "")
        CritCFtable <- data.frame(CritCF = CritCF)
        rownames(CritCFtable) <- paste("k=", ks, sep = "")
        idMaxCritCF <- which(CritCF == max(CritCF))[1]
        if (search != "none")
         {
         	CritCFtable$NumberOfSelectedVars <- NumberOfSelectedVars
         	names(CritCFtable)[2] <- "Number of selected variables"
         	NumberOfSelectedVars <- NumberOfSelectedVars[idMaxCritCF]
         }
        kfin <- ks[idMaxCritCF]
        if (search == "none")
         {
         	res <- list(clustering = clustering, data = data, completeCasesPerc = completeCasesPerc, CritCF = CritCFtable, ks = ks, M = 0, kfin = kfin, method = method, search = search, distance = distance, centPos = centPos, selMetricCent = selMetricCent, initCl = initCl) } else {
          res <- list(clustering = clustering, data = data, completeCasesPerc = completeCasesPerc, CritCF = CritCFtable, ks = ks, M = 0, kfin = kfin, NumberOfSelectedVars = NumberOfSelectedVars, method = method, search = search, maxVars = maxVars, distance = distance, centPos = centPos, selMetricCent = selMetricCent, initCl = initCl) } } else {
        clustering <- list()    
        nimp <- length(usedImp)
        for (imp in 1:nimp)
         {
          clustering[[imp]] <- list()
          cat("Imputation:", imp, "\n")
          aux <- bases[[usedImp[imp]]]
          for (i in 1:nk)
           {
            cat("k =", ks[i], "\n")
            clustering[[imp]][[i]] <- doClusterkmeans(search = search, DataBase = aux, k = ks[i], metricCent = selMetricCent, inertiaPower = 1, maxVars = maxVars, centPos = centPos, initCl = initCl, seed = seed)
           }
          names(clustering[[imp]]) <- paste("k=", ks, sep = "")
         }
        names(clustering) <- paste("imp", usedImp, sep = "")
        CritCF <- matrix(nrow = nimp, ncol = nk)
        rownames(CritCF) <- paste("imp", usedImp, sep = "")
        colnames(CritCF) <- ks
        if (search != "none")
         NumberOfSelectedVars <- CritCF
        for (i in 1:nimp)
         {
          for (j in 1:nk)
           {
   	        CritCF[i, j] <- clustering[[i]][[j]]$CritCFmax
   	        if (search != "none")
   	         NumberOfSelectedVars[i, j] <- length(clustering[[i]][[j]]$SelectedVariables)
   	       }
          }
        IdOptimk <- apply(CritCF, 1, FUN=function(x) which(x==max(x))[1])
        Optimk <- ks[IdOptimk]
        selectedkdistribution <- rep(0, nk)
        names(selectedkdistribution) <- ks
        aux <- 100*summary(as.factor(Optimk))/M
        selectedkdistribution[names(aux)] <- aux
        selcol <- as.numeric(which(selectedkdistribution == max(selectedkdistribution))[1])
        kfin <- ks[selcol]
        rownames(CritCF) <- paste("imp", usedImp, sep="")
        colnames(CritCF) <- paste("k=", ks, sep="")
        clusters <- list()
        kappas <- matrix(nrow = nimp - 1, ncol = nk)
        for (j in 1:nk)
         {
          clusters[[j]] <- matrix(nrow = n, ncol = nimp)
          clusters[[j]][, 1] <- clustering[[1]][[paste("k=", ks[j], sep = "")]]$ClusterVector
          clustering[[1]][[paste("k=", ks[j], sep = "")]]$ClusterVector <- NULL
          for (i in 2:nimp)
           {
           	auxCluster <- clustering[[i]][[paste("k=", ks[j], sep = "")]]$ClusterVector
           	aux <- relabelClusters(refCluster = clusters[[j]][, 1], Cluster = auxCluster)
           	clusters[[j]][, i] <- aux$newCluster
           	kappas[i-1, j] <- aux$kappa
           	clustering[[i]][[paste("k=", ks[j], sep = "")]]$ClusterVector <- NULL
           }
        clusters[[j]] <- as.data.frame(clusters[[j]])
        names(clusters[[j]]) <- paste("imp", usedImp, sep = "")
        clusters[[j]]$assigned <- apply(clusters[[j]],
                                          1,
                                          FUN = function(x) {
        	                                                 aux <- table(as.numeric(x))
        	                                                 aux1 <- as.numeric(aux)
        	                                                 idw <- which(aux1 == max(aux1))
        	                                                 if (length(idw) > 1)
        	                                                  idw <- idw[sample(x = 1:length(idw), size = 1)]
        	                                                 as.numeric(names(aux)[idw])
        	                                                })
         }
        kappas <- as.data.frame(kappas)
        rownames(kappas) <- paste("imp", usedImp[-1], sep = "")
        colnames(kappas) <- paste("k=", ks, sep = "")
        names(clusters) <- paste("k=", ks, sep = "")
        clustering$clusters <- clusters
        clustering$kappas <- kappas
        if (search == "none")
         {
         	res <- list(clustering = clustering, data = data, ks = ks, M = M, usedImp = usedImp, kfin = kfin, CritCF = CritCF, selectedkdistribution = selectedkdistribution, method = method, search = search, distance = distance, centPos = centPos, selMetricCent = selMetricCent, initCl = initCl) } else {
          res <- list(clustering = clustering, data = data, ks = ks, M = M, usedImp = usedImp, kfin = kfin, CritCF  = CritCF, NumberOfSelectedVars = NumberOfSelectedVars, selectedkdistribution = selectedkdistribution, method = method, search = search, maxVars = maxVars, distance = distance, centPos = centPos, selMetricCent = selMetricCent, initCl = initCl)
         }
       }
     }
  class(res) <- "miclust"
  res
 }
