plot.miclust <-
function(x, k = NULL, ...)
 {
  if (!inherits(x, "miclust"))
     stop("argument 'x' must be of class 'miCluster'")
  
  if (x$M == 0)
   {
   	cat("\n")
    cat("   Nothing to plot for a single data set analysis")
    cat("\n") } else {
    	
    if (class(k) != "NULL" && !is.na(k) && (((length(k) != 1L) || (k < 2) || (floor(k) != ceiling(k)))))
   	   stop("the number of clusters, 'k' must be a integer greater than 1")

    kmax <- max(x$ks)
    if (class(k) != "NULL" && !is.na(k) && (k > kmax))   	  
       stop(paste("the number of clusters, 'k' can't be greater than the maximum explored number of clusters,", max(x$ks)))
     
    if (class(k) == "NULL" || is.na(k))
       k <- x$kfin
   
    y <- 100*x$selectedkdistribution / sum(x$selectedkdistribution)
    ymax <- max(y)
  
    if (x$search == "none")
     {
   	  par(oma=c(0,0,1,0), mfrow=c(1, 2), las=1)
      barplot(y, main = "", names.arg = paste("k", x$ks, sep = "="), xlab="Number of clusters", ylab="Selection frequency (%)", ylim=c(0, ymax))
      boxplot(x$CritCF, col="gray", main = "", xlab = "Number of clusters", ylab = "CritCF") } else {	
      par(oma=c(0,0,1,0), mfrow=c(2, 2), las=1)
      barplot(y, main = "", names.arg = paste("k", x$ks, sep = "="), xlab="Number of clusters", ylab="Selection frequency (%)", ylim=c(0, ymax))
      boxplot(x$CritCF, col="gray", main="", xlab = "Number of clusters", ylab = "CritCF")
      SelectedVars <- list()
      NumberOfSelectedVariables <- vector()
      m <- length(x$usedImp)
      for (i in 1:m)
       {
        SelectedVars[[i]] <- x$clustering[[i]][[paste("k=", k, sep="")]]$SelectedVariables
        NumberOfSelectedVariables[i] <- length(SelectedVars[[i]])
       }
      SelectedVars <- as.factor(unlist(SelectedVars))
      SelectedVarsSummary <- as.data.frame(sort(summary(SelectedVars), decreasing=TRUE))
      SelectedVarsSummary <- SelectedVarsSummary*100/m 
      xmin <- min(SelectedVarsSummary)
      xmax <- max(SelectedVarsSummary)
      nv <- dim(SelectedVarsSummary)[1]
      medianNumber <- median(NumberOfSelectedVariables)
      meanNumber <- mean(NumberOfSelectedVariables)
      aux <- quantile(NumberOfSelectedVariables, probs=c(1/4, 3/4))
      Q1number <- aux[1]
      Q3number <- aux[2]
      aux <- table(NumberOfSelectedVariables)
      aux <- 100*aux/sum(aux)
      barplot(aux, main="", xlab=paste("Number of selected variables for", k, "clusters"), ylab="Frequency of selection (%)")
      legend("topright", legend=paste(c("mean =", "median =", "sd =", "Q1 =", "Q3 ="), round(c(meanNumber, medianNumber, sd(NumberOfSelectedVariables), Q1number, Q3number), 1)), cex=0.6)
      plot(c(xmin, xmax), c(1, nv), type="n", yaxt="n", main="", xlab=paste("Variable selection frequency for", k, "clusters (%)"), ylab="", xlim=c(xmin, xmax))
      axis(2, at=1:nv, rownames(SelectedVarsSummary)[nv:1], cex.axis=0.6)
      for (i in 1:nv)
       segments(0, i, SelectedVarsSummary[nv+1-i, 1], i, lwd=1.5)
      abline(h=nv + 1 - Q1number, lty=3)
      abline(h=nv + 1 - medianNumber, lty=2)
      abline(h=nv + 1 - meanNumber, lty=4)
      abline(h=nv + 1 - Q3number, lty=3)
      legend("bottomright", title="Number of selected variables", legend=c("1st and 3rd quartiles", "median", "mean"), lty=c(3, 2, 4), cex=0.6)
      l <- ifelse(sum(x$usedImp) == "all", x$M, length(x$usedImp))
      title(paste("Between-imputations results using", m, "imputed datasets"), outer = TRUE)
     }
  }
}
