summary.miclust <-
function(object, k = NULL, quantileVars = NULL, ...)
 {
  if(!inherits(object, "miclust"))
     stop("argument 'object' must be of class 'miclust'")
     
  if (class(k) != "NULL" && !is.na(k) && (((length(k) != 1L) || (k < 2) || (floor(k) != ceiling(k)))))
   	stop("the number of clusters, 'k' must be a integer greater than 1")

  kmax <- max(object$ks)
  if (class(k) != "NULL" && !is.na(k) && (k > kmax))   	  
     stop(paste("the number of clusters, 'k' can't be greater than the maximum explored number of clusters,", kmax))
     
  if (class(k) == "NULL" || is.na(k))
   k <- object$kfin
   
  if (class(quantileVars) != "NULL" && !is.na(quantileVars) && (((length(quantileVars) != 1L) || (quantileVars <= 0) || (quantileVars > 1))))
   	  stop("the argument 'quantileVars' must be in (0, 1]")
   	  
  if ((object$search != "none") && (class(quantileVars) == "NULL" || is.na(quantileVars)))
     quantileVars <- 0.5
     
  if (object$M == 0)
   {
    if (object$search == "none")
     {
      selectedVariables = colnames(object$data$rawData)
      cluster <- object$clustering[[paste("k=", k, sep="")]]$ClusterVector } else {
      SelectedVars <- object$clustering[[paste("k=", k, sep="")]]$SelectedVariables
      cutPoint <- round(quantile(object$NumberOfSelectedVars, probs = quantileVars))
      if (cutPoint == 0)
       stop(paste("No variables selected if 'quantileVars' is", quantileVars)) 
      selectedVariables <- SelectedVars[1:cutPoint]
      dat <- object$data$rawData[, selectedVariables]
      if (class(dat) == "numeric")
       {
       	dat <- as.data.frame(dat)
       	names(dat) <- selectedVariables
       }
      idcc <- complete.cases(dat)
      dat <- dat[idcc, ]
      n <- nrow(object$data$rawData)
      cluster <- rep(NA, n)
      mod <- kcca(x = dat, k = k, family = object$selMetricCent, simple = T)
      cluster[idcc] <- mod@cluster	
     }
    object$data$rawData$cluster <- cluster
   } else {
    M <- length(object$data$impData)
    n <- nrow(object$data$rawData)
    if (object$search != "none")
     {
      SelectedVars <- list()
      NumberOfSelectedVariables <- vector()
      m <- length(object$usedImp)
      for (i in 1:m)
       {
        SelectedVars[[i]] <- object$clustering[[i]][[paste("k=", k, sep="")]]$SelectedVariables
        NumberOfSelectedVariables[i] <- length(SelectedVars[[i]])
       }
      SelectedVars <- as.factor(unlist(SelectedVars))
      selectedVarsSummary <- as.data.frame(sort(summary(SelectedVars), decreasing=TRUE))
      selectedVarsSummary <- selectedVarsSummary*100/m 
      tab <- data.frame(Variable = rownames(selectedVarsSummary), FrequencyPercent = selectedVarsSummary[ , 1])
      names(tab)[2] <- "Presence(%)"
      cutPoint <- round(quantile(NumberOfSelectedVariables, probs = quantileVars))
      if (cutPoint == 0)
       stop(paste("No variables have been selected if 'quantileVars' is", quantileVars)) 
      selectedVariables <- as.character(tab$Variable[1:cutPoint]) } else {
      selectedVariables <- names(object$data$rawData)
     }
    aux <- assignProbAndKappas(variables = selectedVariables, k = k, metricCent = object$selMetricCent, data = object$data$impData, initialCluster = object$clustering$clusters[[paste("k=", k, sep = "")]]$assigned)
    classMatrix <- aux$classMatrix
    clusterVectors <- aux$clusterVectors
    kappas <- aux$kappas
    kappaDistribution <- quantile(kappas, probs=c(0.025, 0.25, 0.5, 0.75, 0.975))
    kappaDistribution <- c(kappaDistribution[1:3], mean(kappas), kappaDistribution[4:5])
    names(kappaDistribution)[4] <- "mean"
    aux <- classMatrix
    assigned <- rep(NA, n)
    for (i in 1:n)
     {
      x <- aux[i, ]
      assig <- which(x == max(x))
      if (length(assig) > 1)
       {
   	    l <- length(assig)
        sel <- sample(1:l, 1)
        assig <- assig[sel]
       }
      assigned[i] <- assig
     }
    aux$prob <- apply(aux, 1, max)
    aux$assigned <- assigned
    aux <- aux[, c("assigned", "prob")]
    aux <- summaryBy(prob ~ assigned, data=aux, FUN=function(x) quantile(x, probs=0:4/4))
    aux <- aux[, -1]
    rownames(aux) <- paste("Assigned to cluster", 1:nrow(aux))
    colnames(aux) <- c("min", paste("Q", 1:3, sep=""), "max")
    allocationProbabilities <- aux
    object$data$rawData$cluster <- assigned 
    aux <- data.frame(matrix(0, nrow = M, ncol = k))
    for (i in 1:M)
     {
   	  v <- clusterVectors[, i]
   	  object$data$impData[[i]]$cluster <- v
   	  for (j in 1:k)
   	   aux[i, j] <- sum(v == j) 
     }
    clustersSize <- t(apply(aux, 2, FUN=function(x) quantile(x, probs=c(0, 0.025, 0.25, 0.5, 0.75, 0.975, 1))))  
    clustersSize <- round(cbind(clustersSize[, 1:4], apply(aux, 2, mean), clustersSize[, 5:7]), 1)
    colnames(clustersSize)[c(1, 5, 8)] <- c("min.", "mean", "max.")
    imputedClusterSize <- rep(0, k)
    names(imputedClusterSize) <- 1:k 
    auximpcl <- summary(as.factor(assigned))
    imputedClusterSize[names(auximpcl)] <- auximpcl
    clustersSize <- cbind(imputedClusterSize, clustersSize)
    colnames(clustersSize)[1] <- "size"
    rownames(clustersSize) <- paste("cluster", 1:k)
   }
  aux <- object$data$rawData[, c(selectedVariables, "cluster")]
  aux <- aux[complete.cases(aux), ]
  aux1 <- aux
  aux1$cluster <- NULL
  aux1 <- as.data.frame(is.na(aux1))
  miss <- 100*apply(aux1, 2, mean)
  aux1$cluster <- aux$cluster
  aux1 <- aux1*1
  aux1 <- 100*t(summaryBy(. ~ cluster, data=aux1, FUN = mean))[-1, ]
  if (class(aux1) == "numeric")
   {
    aux1 <- t(as.matrix(aux1))
    rownames(aux1) <- selectedVariables
   }
  colnames(aux1) <- paste("%miss.(cl.", colnames(aux1), ")", sep = "")
  aux1 <- as.data.frame(aux1)
  aux1 <- cbind(miss, aux1)
  names(aux1)[1] <- "%miss."
  aux2 <- t(summaryBy(. ~ cluster, data=aux, FUN = function(x) mean(x, na.rm = TRUE)))[-1, ]
  if (class(aux2) == "numeric")
   {
    aux2 <- t(as.matrix(aux2))
    rownames(aux2) <- selectedVariables
   }
  colnames(aux2) <- paste("mean (cl.", colnames(aux2), ")", sep = "")
  aux2 <- as.data.frame(aux2)
  aux3 <- t(summaryBy(. ~ cluster, data=aux, FUN = function(x) sd(x, na.rm = TRUE)))[-1, ]
  if (class(aux3) == "numeric")
   {
    aux3 <- t(as.matrix(aux3))
    rownames(aux3) <- selectedVariables
   }
  colnames(aux3) <- paste("sd (cl.", colnames(aux3), ")", sep = "")
  aux3 <- as.data.frame(aux3)
  aux <- cbind(aux2, aux3)
  rownames(aux) <- selectedVariables
  id1 <- rep(1:k, each = 2)
  id2 <- rep(c(0, k), k)
  id <- id1 + id2
  summaryByCluster <- cbind(aux1, aux[, id])
  
  if (object$M == 0)
   {
    res <- list(M = object$M, completeCasesPerc = object$completeCasesPerc, k = k, selectedVariables = selectedVariables, clusterVector = cluster, summaryByCluster = summaryByCluster, quantileVars = quantileVars,  search = object$search) } else {
   if (object$search == "none")
    {
     res <- list(M = M, k = k, classMatrix = classMatrix, clusterVectors = clusterVectors, kappas = kappas, kappaDistribution = kappaDistribution, cluster = assigned, clustersSize = clustersSize, allocationProbabilities = allocationProbabilities, summaryByCluster = summaryByCluster, usedImp = object$usedImp, search = object$search) } else {
     res <- list(M = M, k = k, quantileVars = quantileVars, selectedVarsPresence = tab, selectedVariables = selectedVariables, classMatrix = classMatrix, clusterVectors = clusterVectors, kappas = kappas, kappaDistribution = kappaDistribution, cluster = assigned, clustersSize = clustersSize, allocationProbabilities = allocationProbabilities, summaryByCluster = summaryByCluster, usedImp = object$usedImp, search = object$search)
     }
   }
  class(res) <- "summary.miclust"
  res  
 }
