\name{getData}
\alias{getData}
\title{Function for create a \code{miData} object.}
\description{This function creates an object of class \code{miData} to be clustered by the function \code{miclust}.}
\usage{
getData(data)
}
\arguments{
  \item{data}{a \code{list} or \code{data.frame} object. If it is a data frame, it is assumed to contain the raw data, with or without missing data, and without imputations. If it is a list of data frames, it is assumed that the first element contains the raw data and the remaining ones correspond to multiple imputed datasets. Since all variables are considered in the clustering procedure, no identifier variables must be present in the data.}
}

\value{
  \item{rawData}{a data frame containing the raw data.}
  \item{impData}{if \code{data} is an object of class \code{list}, \code{impData} is a list containing the standardized imputed datasets. This standardization is performed by recoding binary variables to 0/1, centering all variables at their mean and standardizing continuous variables by their standard deviation.}
}

\author{Jose Barrera-Gomez and Xavier Basagana.}

\seealso{\code{\link{miclust}}}

\examples{
# data minhanes:
data(minhanes)
class(minhanes)

# number of imputed datasets:
length(minhanes) - 1

# raw data with missing values:
summary(minhanes[[1]])

# first imputed dataset:
minhanes[[2]]
summary(minhanes[[2]])

# data preparation for a complete case cluster analysis:
data1 <- getData(minhanes[[1]])

class(data1)
names(data1)
# no imputed datasets:
data1$impData

# data preparation for a multiple imputation cluster analysis:
data2<- getData(minhanes)

class(data2)
names(data2)
# number of imputed datasets:
length(data2$impData)

# imputed datasets are standardized:
summary(data2$rawData)
summary(data2$impData[[1]])
}

