\name{getVariablesFrequency}
\alias{getVariablesFrequency}
\title{Function to get the ranked selection frequency of the variables.}
\description{This function creates the ranked selection frequencies of all the variables that have been selected at least once along the analyzed imputed datasets. The function can be useful for customizing the plot of these frequencies as it is shown in `Examples'.}
\usage{
getVariablesFrequency(x, k = NULL)
}
\arguments{
  \item{x}{an object of class \code{miclust} obtained with the function \code{miclust}.}
  \item{k}{the number of clusters. The default value is the optimal number of clusters obtained by the function \code{miclust}.}
}

\value{
  \item{percFreq}{vector of the selection frequencies (percentage of times) of the variables in decreasing order.}
  \item{varNames}{names of the variables.}
}

\author{Jose Barrera-Gomez and Xavier Basagana.}

\seealso{\code{\link{miclust}}}

\examples{
require(graphics)
data(minhanes)

# Data preparation:

minhanes1 <- getData(data = minhanes)

# Using only the imputations 1 to 10 for the clustering process and exploring 2 vs. 3 clusters:

minhanes1clust <- miclust(data = minhanes1, search = "backward", ks = 2:3, usedImp = 1:10, seed = 4321)
minhanes1clust
minhanes1clust$kfin # Optimal number of clusters

# Obtaining the selection frequency of the variables for the optimal number of clusters:

y <- getVariablesFrequency(minhanes1clust)
y

# Plot:

plot(y$percFreq, type = "h", main = "", xlab = "Variable", ylab = "Percentage of times selected",
      xlim = 0.5 + c(0, length(y$varNames)), lwd = 15, col = "blue", xaxt = "n")
axis(1, at = 1:length(y$varNames), labels = y$varNames)
}
