\name{summary.miclust}
\alias{summary.miclust}
\alias{print.summary.miclust}

\title{Within-cluster descriptive analysis}

\description{This function performs a within-cluster descriptive analysis of the variables after the clustering process performed by the function \code{\link{miclust}}.}

\usage{
\method{summary}{miclust}(object, k = NULL, quantileVars = NULL, ...)
\method{print}{summary.miclust}(x, ..., digits = 2)
}
\arguments{
  \item{object}{an object of class \code{miclust} obtained by the function \code{\link{miclust}}.}
  \item{x}{an object of class \code{summary.miclust} obtained by the function \code{summary.miclust}.}
  \item{k}{the number of clusters. The default value is the optimal number of clusters obtained by \code{\link{miclust}}.}
  \item{quantileVars}{numeric. If a variable selection procedure was used, the cut-off percentile in order to decide the number of selected variables in the variable reduction procedure by decreasing order of presence along the imputations results. The default value is \code{quantileVars} = 0.5, i.e., the number of selected variables is the median number of selected variables along the imputations.}
  \item{digits}{minimal number of significant digits.}
  \item{\dots}{further arguments for the summary function.}
}

\value{
  \item{M}{number of imputations used in the descriptive analysis which is the total number of imputations provided.}
  \item{k}{number of clusters.}
  \item{summaryByCluster}{within-cluster descriptive analysis of the selected variables.}
  \item{search}{search algorithm for the selection variable procedure.}
  \item{cluster}{vector containing the individuals cluster assignments.}
  \item{completeCasesPerc}{if a single dataset have been clustered, the percentage of complete cases in the dataset.}
  \item{selectedVariables}{if a single dataset have been clustered, the selected variables obtained considering \code{quantileVars}.}
  \item{classMatrix}{if imputations were analyzed, the individual probabilities of cluster assignment.}
  \item{clusterVectors}{if imputations were analyzed, the individual cluster assignment in each imputation.}
  \item{kappas}{if imputations were analyzed, the Cohen's kappa values after comparing the cluster vector in the first imputation with the  cluster vector in each of the remaining imputations.}
  \item{kappaDistribution}{a summary of \code{kappas}.}
  \item{clustersSize}{if imputations were analyzed, size of the imputed cluster and between-imputations summary of the cluster size.}
  \item{allocationProbabilities}{if imputations were analyzed, descriptive summary of the probability of cluster assignment.}
  \item{summaryByCluster}{if imputations were analyzed, within-cluster descriptive analysis of the selected variables.}
  \item{usedImp}{imputations used in the clustering procedure.}
  \item{quantileVars}{if imputations were analyzed and variable selection was performed, the input value of \code{quantileVars}.}
  \item{selectedVarsPresence}{if imputations were analyzed and variable selection was performed, the presence of the selected variables along imputations.}
  \item{selectedVariables}{if imputations were analyzed and variable selection was performed, the names of the selected variables.}
}

\references{Basagana X, Barrera-Gomez J, Benet M, Anto JM, Garcia-Aymerich J. A framework for multiple imputation in cluster analysis. American Journal of Epidemiology. 2013;177(7):718-25.}

\author{Jose Barrera-Gomez and Xavier Basagana.}

\seealso{\code{\link{miclust}}.}